#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import scipy.stats as stats
import seaborn as sns
import matplotlib.pyplot as plt
from itertools import combinations

def get_options():
    parser = argparse.ArgumentParser(description="The output files of the Mann-Whitney plate level analysis and the Z score analysis are inputted. The files are tested to see which conditions fail at certain thresholds of Normality and Mann-Whitney p value.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-iz", "--InputFile_Z_Score", help="output file from Z_score_count.py")
    parser.add_argument("-imwp", "--InputFile_MWP", help="output file from Mann_Whitney_Plate_Level.py")
    parser.add_argument("-oz", "--OutputFile_Z_Score", help="A CSV file showing the plates and the thresholds at which they pass and fail for the Z-score test. Here normality percentages which are lower than the threshold tested fail.")
    parser.add_argument("-omwp", "--OutputFile_MWP", help="A CSV file showing the plates and the thresholds at which they pass and fail for the Mann-Whitney test. Here p values which are lower than the threshold tested fail.")
    parser.add_argument("-mo", "--Merged_Outputfile", help="A CSV file showing the plates and the thresholds at which they pass and fail for both.")
    return parser.parse_args()


def main():
    options = get_options()
    inptZ = options.InputFile_Z_Score
    otputZ = options.OutputFile_Z_Score
    inptMWP = options.InputFile_MWP
    otputMWP = options.OutputFile_MWP
    otpt_merge = options.Merged_Outputfile
    #### Calculating the Normality threshold Pass and Fail values
    zero_count = pd.read_csv(inptZ)
    zero_count = zero_count.set_index(['Condition','Plate','Replicate','Batch'])
    abnorm_p_f = pd.DataFrame(columns=['Filename','Condition','Plate','Replicate','Batch',
                                   'Normality Threshold 20%','Normality Threshold 30%',
                                   'Normality Threshold 40%','Normality Threshold 50%'
                                   ,'Normality Threshold 60%','Normality Threshold 80%'])

    z_count_array = np.array(zero_count)
    #Checks percentage normality for each condition lower than thresholds
    #  and marks as F for fail or greater than threshold and marks as P for pass 
    # and creates dataset of Ps and Fs for the various thresholds
    plt = {x[0:5] for x in zero_count.index}
    for r,p in zip(range(len(z_count_array)),sorted(plt)):
        print(z_count_array[r][4],z_count_array[r][5])
        if z_count_array[r][4] < 20:
            PF20 = 'F' 
        if z_count_array[r][4] < 30:
            PF30 = 'F'
        if z_count_array[r][4] < 40:
            PF40 = 'F' 
        if z_count_array[r][4] < 50:
            PF50 = 'F'
        if z_count_array[r][4] < 60:
            PF60 = 'F' 
        if z_count_array[r][4] < 80:
            PF80 = 'F'
        if z_count_array[r][4] > 20:
            PF20 = 'P' 
        if z_count_array[r][4] > 30:
            PF30 = 'P'
        if z_count_array[r][4] > 40:
            PF40 = 'P' 
        if z_count_array[r][4] > 50:
            PF50 = 'P'
        if z_count_array[r][4] > 60:
            PF60 = 'P' 
        if z_count_array[r][4] > 80:
            PF80 = 'P'
        name = (p[0]+'-'+str(p[1])+"-"+p[3].replace("Batch","")+'_'+p[2]+'.JPG.iris',p[0]
                ,p[1],p[2],p[3], PF20,PF30,PF40,PF50,PF60,PF80)
        columns = list(abnorm_p_f)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        abnorm_p_f = abnorm_p_f.append(data, True)
    abnorm_p_f.to_csv(otputZ, index=False)
#### Calculating the Mann_Whitney P-Value threshold Pass and Fail values   
    Pmean = pd.read_csv(inptMWP)
    Pmean = Pmean.set_index(['Condition','Plate','Replicate','Batch'])
    thres_ar = [x for x in sorted(Pmean['Mean P-Value']) if x > 0]
    # takes thresholds at intervals across the achieved values
    thres_array = (sorted(thres_ar)[(int(len(thres_ar)*(1/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(4/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(8/24)))],
                   sorted(thres_ar)[int((len(thres_ar)*(12/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(16/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(20/24)))])
    
    mwp_p_f = pd.DataFrame(columns=["Filename",'Condition','Plate','Replicate','Batch',
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[0], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[1], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[2], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[3], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[4], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[5], precision=3))])
    Pmean_array = np.array(Pmean)
    plt2 = {x[0:5] for x in Pmean.index}
    #Checks mean p-value for each condition lower than thresholds
    #  and marks as F for fail or greater than threshold and marks as P for pass 
    # and creates dataset of Ps and Fs for the various thresholds
    for r,p in zip(range(len(Pmean_array)),sorted(plt2)):
        if Pmean_array[r][1] < thres_array[5]:
            PF5 = 'F' 
        if Pmean_array[r][1] < thres_array[4]:
            PF4 = 'F' 
        if Pmean_array[r][1] < thres_array[3]:
            PF3 = 'F'
        if Pmean_array[r][1] < thres_array[2]:
            PF2 = 'F' 
        if Pmean_array[r][1] < thres_array[1]:
            PF1 = 'F'
        if Pmean_array[r][1] < thres_array[0]:
            PF0 = 'F' 
        if Pmean_array[r][1] > thres_array[5]:
            PF5 = 'P'
        if Pmean_array[r][1] > thres_array[4]:
            PF4 = 'P' 
        if Pmean_array[r][1] > thres_array[3]:
            PF3 = 'P'
        if Pmean_array[r][1] > thres_array[2]:
            PF2 = 'P' 
        if Pmean_array[r][1] > thres_array[1]:
            PF1 = 'P'
        if Pmean_array[r][1] > thres_array[0]:
            PF0 = 'P' 
        name = ((p[0]+'-'+str(p[1])+"-"+p[3].replace("Batch","")+'_'+p[2]+'.JPG.iris'),p[0]
                ,p[1],p[2],p[3], PF0, PF1,PF2,PF3,PF4,PF5)
        columns = list(mwp_p_f)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        mwp_p_f = mwp_p_f.append(data, True)                              
    mwp_p_f = mwp_p_f.sort_values(['Condition','Plate'])
    mwp_p_f = mwp_p_f.reset_index(drop=True)
    mwp_p_f.to_csv(otputMWP, index=False)
    
    #Merges the two datasets together and shows percentage fails across both tests
    p_f_merge = pd.merge(mwp_p_f,abnorm_p_f)
    p_f_merge['Percentage Fails'] = 'N'
    for index, row in p_f_merge.iterrows():
        P = 0 
        F = 0
        for cols in row:
            if cols == 'P':
                P = P +1
            if cols == 'F':
                F = F +1
        p_f_merge.iloc[index]['Percentage Fails'] = (F/(F+P))*100
    p_f_merge_ind = p_f_merge.set_index(['Condition','Plate'])
    plt4 = {x[0:2] for x in p_f_merge_ind.index}
    p_f_merge_reps = pd.DataFrame(columns = ['Filename', 'Condition', 'Plate', 'Replicate', 'Batch',
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[0], precision=3)),
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[1], precision=3)),
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[2], precision=3)),
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[3], precision=3)),
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[4], precision=3)),
           'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[5], precision=3)),
           'Normality Threshold 20%', 'Normality Threshold 30%',
           'Normality Threshold 40%', 'Normality Threshold 50%',
           'Normality Threshold 60%', 'Normality Threshold 80%',
           'Percentage Fails','Replicates within Condition/Plate'])
    for p4 in sorted(plt4):
        dfpfm = p_f_merge_ind.xs((p4), axis =0, drop_level=False)
        dfpfm['Replicates within Condition/Plate'] = "N"
        replist = dfpfm['Replicate'].values
        dfpfm = dfpfm.reset_index()
        first_column = dfpfm.pop('Filename')
        dfpfm.insert(0, 'Filename', first_column)
        for index, row in dfpfm.iterrows():
            dfpfm.loc[:,('Replicates within Condition/Plate')] = str(replist)
        p_f_merge_reps = p_f_merge_reps.append(dfpfm)
    p_f_merge_reps.to_csv(otpt_merge, index=False)
    return p_f_merge_reps

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import random
from scipy import spatial
import re
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.patches as  mpatches
from sklearn.metrics import auc
import numpy as np
import os
import itertools as it 
def get_options():
    parser = argparse.ArgumentParser(description="Calculates the cosine similarity scores for the phenotypic profiles of genes from the same operon and genes from different operons. Produces a density plot of the cosine similarity scores for genes of the same and different operons. Produces an ROC curve testing models ability at different threshold. ",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The dataset with gene names added. Output from Add_gene_names.py")
    parser.add_argument("-o", "--OutputFile", help="List of genes compared and the cosine similarity score as well as if they belong to the same operon")
    parser.add_argument("-or", "--Output_ROC_curve", help="Plot of the ROC curve and AUC score.")
    parser.add_argument("-od", "--Output_Density_plot", help="Density plot of the cosine similarity scores for same and different operons.")
    parser.add_argument("-clus", "--Cluster_file", help="A CSV file containing the operon clusters for each gene within the bacterium of interest, where columns = (Cluster,Gene). The Genes must match the names assigned to the scored dataset.")
    return parser.parse_args()

def main():
    options = get_options()
    inpt = options.InputFile
    outpt = options.OutputFile
    Op_Clus = options.Cluster_file
    outroc = options.Output_ROC_curve
    outdens = options.Output_Density_plot
    outroc = os.path.expanduser(outroc)
    outdens = os.path.expanduser(outdens)
    #Open the operon cluster and s-score files
    df = pd.read_csv(Op_Clus)
    df = df[['Gene', 'Cluster']]
    ar1 = np.array(df)
    df_14 = pd.read_table(inpt, index_col=0)
    df_14 = df_14.reset_index()
    df_14 = df_14.rename(columns={"index": "Gene"})
    df_14 = df_14.sort_values('Gene')
    df_14 = df_14[~df_14.Gene.str.contains("empty")]
    df_14 = df_14[~df_14.Gene.str.contains("EMPTY")]
    df_14 = df_14.reset_index(drop=True)
    ar_14 = np.array(df_14)
    
    #Collect only the clusters for genes in the dataset and assigning clusters to the dataset in end column
    df_Clus = pd.DataFrame(columns=df.columns)
    for i in range(len(ar1)): 
        x = ar1[i][0]
        if str(x) in ar_14[:,0]:
            rows = df.loc[i]
            df_Clus = df_Clus.append(rows, ignore_index=True)
    df_merge = pd.merge(df_14,df_Clus)
    df_co_sim_R_T = pd.DataFrame()
    ar_merge = np.array(df_merge)
    mainlist = list(range(0,len(df_merge)))
    for i,j in it.combinations(mainlist, 2):
        #ignores comparison with self
        if i != j:
            #if in same cluster list all s-scores
            if str(ar_merge[i][-1]) == str(ar_merge[j][-1]):
                p = list(ar_merge[i,1:-1])
                p2 = list(ar_merge[j,1:-1])
                data1 = []
                data2 = []
                #ensures nans are removed as well as the corresponding value for the other list.
                for itemp,indexp,itemp2, in zip(p,range(len(p)),p2):
                    if str(itemp) != "nan" and str(itemp2) != "nan":
                        data1.append(float(itemp))
                        data2.append(float(itemp2))
                n = str(ar_merge[i,0:1])
                n2 = str(ar_merge[j,0:1])
                #calculates cosine similarity between the phenotypic profiles of s-scores for the two genes and adds to a df.
                cosine_similarity = 1 - spatial.distance.cosine(data1, data2)
                name = np.array([n, n2, cosine_similarity, "TRUE"],dtype=object)
                df_co_sim_R_T = df_co_sim_R_T.append(pd.DataFrame(name).T)
    df_co_sim_R_T.columns=['Gene1','Gene2','Cosine_score','Same Operon']
    df_co_sim_R_T = df_co_sim_R_T.reset_index(drop=True)

    random.seed(2)
    def random_pairs(number_list): 
        return [number_list[i] for i in random.sample(range(len(number_list)), 2)]
    #selects twice as many pairs as needed for different operons as likely some
    # will come up as the same and want to later syphon down to same number of comparisions as for same operon.
    n = (2*len(df_co_sim_R_T))
    numbers = list(range(len(df_merge)))
    rp = [random_pairs(numbers) for i in range(n)]
    df_co_sim_R_F = pd.DataFrame()
    for k in range(n):
            i = rp[k][0]
            j = rp[k][1]
            d1 = list(ar_merge[i,-1:])
            d2 = list(ar_merge[j,-1:])
            #if operon clusters do not match.
            if d1 != d2:
                p = list(ar_merge[i,1:-1])
                p2 = list(ar_merge[j,1:-1])
                data1 = []
                data2 = []
                for itemp,indexp,itemp2, in zip(p,range(len(p)),p2):
                    if str(itemp) != "nan" and str(itemp2) != "nan":
                        data1.append(float(itemp))
                        data2.append(float(itemp2))
                n = ar_merge[i,0:1]
                n2 = ar_merge[j,0:1]
                cosine_similarity = 1 - spatial.distance.cosine(data1, data2)
                name = np.array([n, n2, cosine_similarity, "FALSE"],dtype=object)
                df_co_sim_R_F = df_co_sim_R_F.append(pd.DataFrame(name).T)
    df_co_sim_R_F.columns=['Gene1','Gene2','Cosine_score','Same Operon']
    df_co_sim_R_F = df_co_sim_R_F.reset_index(drop=True)

    # drop any NA values
    df_co_sim_R_T.dropna(subset = ["Cosine_score"], inplace=True)
    df_co_sim_R_F.dropna(subset = ["Cosine_score"], inplace=True)
    df_co_sim_R_F = df_co_sim_R_F.reset_index(drop=True)
    df_co_sim_R_T = df_co_sim_R_T.reset_index(drop=True)
    
    
    if len(df_co_sim_R_F) < len(df_co_sim_R_T):
        # pick random gene pairs from the different operons dataframe
        # selecting same number as there are in the same operon dataframe
        random.seed(2)
        r_S2 = random.sample(range(len(df_co_sim_R_T)), len(df_co_sim_R_F))
        df_S2= pd.DataFrame(columns=df_co_sim_R_T.columns)
        for i in r_S2:
            rows = df_co_sim_R_T.loc[i]
            df_S2 = df_S2.append(rows, ignore_index=True)
        df_T_F = pd.concat([df_co_sim_R_F, df_S2],ignore_index=True)
    elif len(df_co_sim_R_F) > len(df_co_sim_R_T):
        # pick random gene pairs from the different operons dataframe
        # selecting same number as there are in the same operon dataframe
        random.seed(2)
        r_S2 = random.sample(range(len(df_co_sim_R_F)), len(df_co_sim_R_T))
        df_S2= pd.DataFrame(columns=df_co_sim_R_F.columns)
        for i in r_S2:
            rows = df_co_sim_R_F.loc[i]
            df_S2 = df_S2.append(rows, ignore_index=True)
    # Join the subsetted different operon table and same operon table
        df_T_F = pd.concat([df_co_sim_R_T, df_S2],ignore_index=True)
    # Format the merged table so it can be pivoted to 
    # have cosine similarity scores put into a same or different operon column
    df_T_F['Cosine_score'] = df_T_F['Cosine_score'].astype(float)
    for f in df_T_F.columns:
        if f != 'Cosine_score':
            df_T_F[f] = df_T_F[f].astype(str)

    for f in list(['Gene1',"Gene2"]):
        df_T_F[f] = df_T_F[f].str.replace("\[\'",'', regex = True)
        df_T_F[f] = df_T_F[f].str.replace("\'\]",'', regex = True)

    for f in df_T_F.columns:
        if f != 'Cosine_score':    
            df_T_F[f] = pd.array(df_T_F[f].tolist())
    df_T_F.to_csv(outpt, index=False)
    df_T_F_2 = pd.pivot_table(df_T_F, values='Cosine_score', index=['Gene1','Gene2'],columns=['Same Operon'])
    # Produce density plot for similarity scores between same and different operons
    sns.set(style="darkgrid") 
    fig = sns.kdeplot(df_T_F_2['FALSE'], shade=True, color="r")
    fig = sns.kdeplot(df_T_F_2['TRUE'], shade=True, color="b")
    plt.xlabel("Cosine Similarity Score")
    plt.ylabel("Density")
    handles = [mpatches.Patch(facecolor="r", label="Different Operon"), mpatches.Patch(facecolor="b", label="Same Operon")]
    plt.legend(handles=handles)
    fig.set_title('Density Plot of Cosine Similarity Scores Between Genes of the Same and Different Operons')
    plt.savefig(outdens, bbox_inches='tight')
    df_FP = df_T_F_2.reset_index(drop=True)
    F = list(df_FP['FALSE'])
    F = [x for x in F if str(x) != 'nan']
    T = list(df_FP['TRUE'])
    T = [x for x in T if str(x) != 'nan']
    df_thres = pd.DataFrame()
    #calculate true and false positives and negatives at different thresholds for AUC and ROC curve production.
    for i in np.arange(-1, 1.1, 0.1):
        TP = 0
        TN = 0
        FP = 0
        FN = 0
        for row in F:
            if row < i:
                TN = TN + 1
            else: 
                FP = FP + 1
        for row in T:
            if row < i:
                FN = FN + 1
            else: 
                TP = TP + 1
        sens = TP/(TP+FN)
        spec = TN/(TN+FP)
        FPR = 1-spec
        name = np.array([i,sens,spec,FPR],dtype=object)
        df_thres = df_thres.append(pd.DataFrame(name).T)
    df_thres.columns=['Threshold','Sensitivity' ,'Specificity','False Positive Rate']
    sens_rate = list(df_thres['Sensitivity'])
    spec_rate = list(df_thres['False Positive Rate'])
    fig, ax = plt.subplots(figsize=(6,6))
    ax.plot(spec_rate, sens_rate)
    ax.set_xlabel('False Positive Rate')
    ax.set_ylabel('True Positive Rate')
    ax.set_title('ROC curve for Assigning S-scores to Genes of Same and Different Operons')
    AUC_txt = f'AUC {auc(spec_rate, sens_rate)}'
    fig.suptitle(AUC_txt, fontsize=12, fontweight='bold')
    plt.savefig(outroc, bbox_inches='tight')

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import scipy.stats as stats
import seaborn as sns
import matplotlib.pyplot as plt
from itertools import combinations

def get_options():
    parser = argparse.ArgumentParser(description="The output files of the Mann-Whitney condition level analysis and the condition variance analysis are inputted. The files are tested to see which conditions fail at certain thresholds of variance and Mann-Whitney p value.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-iv", "--InputFile_Variance", help="Output file from Condition_Variance.py")
    parser.add_argument("-imwc", "--InputFile_MWC", help="Output file from Mann_Whitney_Condition_Level.py")
    parser.add_argument("-ov", "--OutputFile_Variance", help="A CSV file showing the conditions and the thresholds at which they pass and fail. Here variances which are greater than the threshold tested fail.")
    parser.add_argument("-omwc", "--OutputFile_MWC", help="A CSV file showing the conditions and the thresholds at which they pass and fail. Here p values which are lower than the threshold tested fail.")
    return parser.parse_args()
    

def main():
    options = get_options()
    inptV = options.InputFile_Variance
    otputV = options.OutputFile_Variance
    inptMWC = options.InputFile_MWC
    otputMWC = options.OutputFile_MWC
    #### Calculating the Mann_Whitney P-Value threshold Pass and Fail values   
    Pmean = pd.read_csv(inptMWC)
    Pmean = Pmean.set_index(['Condition','Batch'])
    thres_ar = [x for x in sorted(Pmean['Mean Variance P-Value']) if x > 0]
    # takes thresholds at intervals across the achieved values
    thres_array = (sorted(thres_ar)[(int(len(thres_ar)*(1/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(4/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(8/24)))],
                   sorted(thres_ar)[int((len(thres_ar)*(12/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(16/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(20/24)))])
    
    mwc_p_f = pd.DataFrame(columns=['Condition','Batch',
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[0], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[1], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[2], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[3], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[4], precision=3)),
                                   'Mann_Whitney Threshold '+ str(np.format_float_scientific(thres_array[5], precision=3))])
    Pmean_array = np.array(Pmean)
    plt2 = {x[0:5] for x in Pmean.index}
    print(thres_array[5],thres_array[4],thres_array[3],thres_array[2],thres_array[1],thres_array[0])
    #Checks mean Variance p-value for each condition greater than thresholds
    #  and marks as F for fail or lower than threshold and marks as P for pass 
    # and creates dataset of Ps and Fs for the various thresholds
    for r,p in zip(range(len(Pmean_array)),sorted(plt2)):
        print(Pmean_array[r][1])
        if Pmean_array[r][1] > thres_array[5]:
            PF5 = 'F' 
        if Pmean_array[r][1] > thres_array[4]:
            PF4 = 'F' 
        if Pmean_array[r][1] > thres_array[3]:
            PF3 = 'F'
        if Pmean_array[r][1] > thres_array[2]:
            PF2 = 'F' 
        if Pmean_array[r][1] > thres_array[1]:
            PF1 = 'F'
        if Pmean_array[r][1] > thres_array[0]:
            PF0 = 'F' 
        if Pmean_array[r][1] <= thres_array[5]:
            PF5 = 'P'
        if Pmean_array[r][1] <= thres_array[4]:
            PF4 = 'P' 
        if Pmean_array[r][1] <= thres_array[3]:
            PF3 = 'P'
        if Pmean_array[r][1] <= thres_array[2]:
            PF2 = 'P' 
        if Pmean_array[r][1] <= thres_array[1]:
            PF1 = 'P'
        if Pmean_array[r][1] <= thres_array[0]:
            PF0 = 'P' 
        name = (p[0],p[1],PF0, PF1,PF2,PF3,PF4,PF5)
        columns = list(mwc_p_f)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        mwc_p_f = mwc_p_f.append(data, True)   
    mwc_p_f = mwc_p_f.sort_values(['Condition','Batch'])
    mwc_p_f = mwc_p_f.reset_index(drop=True)
    mwc_p_f.to_csv(otputMWC, index=False)
    
    ave_Var_cond = pd.read_csv(inptV)
    ave_Var_cond = ave_Var_cond.set_index(['Condition','Batch'])
    thres_ar = [x for x in sorted(ave_Var_cond['Average Variance']) if x > 0]
    # takes thresholds at intervals across the achieved values
    thres_array = (sorted(thres_ar)[(int(len(thres_ar)*(1/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(4/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(8/24)))],
                   sorted(thres_ar)[int((len(thres_ar)*(12/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(16/24)))],
                   sorted(thres_ar)[(int(len(thres_ar)*(20/24)))])
    
    varc_p_f = pd.DataFrame(columns=['Condition','Batch',
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[0], precision=3)),
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[1], precision=3)),
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[2], precision=3)),
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[3], precision=3)),
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[4], precision=3)),
                                   'Average Variance Threshold '+ str(np.format_float_scientific(thres_array[5], precision=3))])
    varc_array = np.array(ave_Var_cond)
    plt2 = {x[0] for x in ave_Var_cond.index}
    #Checks mean Variance for each condition greater than thresholds
    #  and marks as F for fail or lower than threshold and marks as P for pass 
    # and creates dataset of Ps and Fs for the various thresholds
    for r,p in zip(range(len(varc_array)),sorted(ave_Var_cond.index)):
        if varc_array [r] > thres_array[0]:
            PF0 = 'F' 
        if varc_array [r] > thres_array[1]:
            PF1 = 'F' 
        if varc_array [r] > thres_array[2]:
            PF2 = 'F'
        if varc_array [r] > thres_array[3]:
            PF3 = 'F' 
        if varc_array [r] > thres_array[4]:
            PF4 = 'F'
        if varc_array [r] > thres_array[5]:
            PF5 = 'F' 
        if varc_array [r] <= thres_array[0]:
            PF0 = 'P'
        if varc_array [r] <= thres_array[1]:
            PF1 = 'P' 
        if varc_array [r] <= thres_array[2]:
            PF2 = 'P'
        if varc_array [r] <= thres_array[3]:
            PF3 = 'P' 
        if varc_array [r] <= thres_array[4]:
            PF4 = 'P'
        if varc_array [r] <= thres_array[5]:
            PF5 = 'P' 
        name = (p[0],p[1], PF0, PF1,PF2,PF3,PF4,PF5)
        columns = list(varc_p_f)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        varc_p_f = varc_p_f.append(data, True)                              
    varc_p_f = varc_p_f.sort_values(['Condition'])
    varc_p_f = varc_p_f.reset_index(drop=True)
    varc_p_f.to_csv(otputV, index=False)

if __name__ == "__main__":
    main()
# %%



#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import scipy.stats as stats

def get_options():
    parser = argparse.ArgumentParser(description="Compares the distributions of the colony sizes of replicate plates of the same condition and determines if replicate plates have the same distribution based on the p value of the Mann whitney test.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The normalised csv file from Check_Normalisation.py output")
    parser.add_argument("-o", "--OutputFile", help="A CSV file with the u statistics and p values for each comparison.")
    parser.add_argument("-o2", "--OutputFile_Mean", help="A CSV file with the mean u statistics and p values for each replicate")
    return parser.parse_args()


def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    outputfile2 = options.OutputFile_Mean
    #opens the normalised dataset file
    m = pd.read_csv(inputfile,
                      index_col=[0, 1],
                      header=[0, 1, 2, 3])
    m.columns = m.columns.swaplevel(2,3)
    plates = {x[0:3] for x in m.columns}
    Mann_whit_all = pd.DataFrame(columns=['Plate','Condition','Batch','Replicate 1',
                                      'Replicate 2','U-statistic', 'P-Value'])
    # splits and iterates by plate, condition.
    for p in sorted(plates):
        print(p)
        df3 = m.xs((p), axis =1, drop_level=False)
        print(df3)
        ar1 = np.array(df3)
        for i in list(range(0,len(df3.columns))):
            for j in list(range(0,len(df3.columns))):
                #compares columns that arent the same in pairs
                if i != j:
                    #takes the whole set of values then removes nan values and the corresponding values in the paired set
                    #This allows the mannwhitneyu to work
                    rep1 = ar1[:,i]
                    rep2 = ar1[:,j]
                    rep1 = rep1[~np.isnan(rep1)]
                    rep2 = rep2[~np.isnan(rep2)] 
                    u_statistic, p_value = stats.mannwhitneyu(rep1,rep2)
                    #sets all into a df for output.
                    name = (df3.columns[i][0],df3.columns[i][1],df3.columns[i][2],
                                df3.columns[i][3], df3.columns[j][3],u_statistic, p_value)
                    columns = list(Mann_whit_all)
                    data = []
                    zipped = zip(columns, name)
                    a_dictionary = dict(zipped)
                    data.append(a_dictionary)
                    Mann_whit_all = Mann_whit_all.append(data, True)
                    
    Mann_whit_all = Mann_whit_all.set_index(['Replicate 1','Plate','Condition','Batch'])
    Mann_whit_all = Mann_whit_all.sort_index(0)
    Mann_whit_all.to_csv(outputfile)
    #averages out the p-values to produce p-value mean for each replicate
    Pmean = Mann_whit_all.groupby(level=[0,1,2,3]).mean()
    Pmean = Pmean.reset_index()
    Pmean = Pmean.rename(columns={"Replicate 1":"Replicate","U-statistic":'Mean U-Stat',"P-Value":'Mean P-Value'})
    Pmean.to_csv(outputfile2,index=False)   
    return Pmean

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd

def get_options():
    parser = argparse.ArgumentParser(description="Outputs a list of conditions which were removed at a certain chosen threshold for the Mann Whitney Condition Level test. Also outputs a new dataset to go back into the process of normalisation and scoring, but with detrimental plates removed.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="output from Mann_Whitney_Condition_Level.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file with the name of the plates that were removed and their file names.")
    parser.add_argument("-od", "--Original_Dataset", help="The original .csv dataset used in the first stage or the output of MW_Plates_to_Remove.py or Z_Plates_to_Remove.py or Variance_Conditions_to_Remove.py to remove more plates")
    parser.add_argument("-or", "--Output_removed", help="A .csv dataset with detrimental plates removed.")
    parser.add_argument("-t", "--Threshold",type=float, help="A chosen threshold, usually based off of the bar chart produced by Bar_plot_Condition.py.")
    return parser.parse_args()
    
def main():
    options = get_options()
    input_V = options.InputFile
    outpt = options.OutputFile
    input_original = options.Original_Dataset
    outpt_removed = options.Output_removed
    threshold = options.Threshold
    #reads the output from Mann_Whitney_Condition_Level.py
    input_DF_2 = pd.read_csv(input_V)
    #reads the original dataset before normalisation
    m = pd.read_csv(input_original,index_col=[0, 1],header=[0, 1, 2, 3])
    #swaps levels so later we can remove by just condition and batch
    m.columns = m.columns.swaplevel(0,1)
    m.columns = m.columns.swaplevel(1, 3)
    Plts_rm = pd.DataFrame(columns=['Condition','Batch',
                                    'Mean Variance P-Value','Mean Variance U-Stat'])
    #goes down row by row and checks if mean variance p-value is larger than the chosen threshold,
    # if so it is appended to a row of Plts_rm dataframe
    lst = []
    for row in input_DF_2.iterrows():
        if row[1][3] > threshold:
            #appends the condition and batch name to a list
            abc=(str(row[1][0]),row[1][1])
            lst.append(abc)
            name = (row[1][0],row[1][1],row[1][3],row[1][2])
            columns = list(Plts_rm)
            data = []
            zipped = zip(columns, name)
            a_dictionary = dict(zipped)
            data.append(a_dictionary)
            Plts_rm = Plts_rm.append(data, True)
    Plts_rm.to_csv(outpt)
    #checks if condition and batch in the original 
    # dataset supplied as you can supply a dataset that has had things removed already from another test.
    lst2 =  {x[0:2] for x in m.columns}
    lst3 = []
    for i in lst:
        if i in lst2:
            lst3.append(i)
    #drops columns which match the condition and batch.
    n = m.drop(columns=lst3, axis=1)
    n.columns = n.columns.swaplevel(3,1)
    n.columns = n.columns.swaplevel(1,0)
    n.to_csv(outpt_removed)
    
if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import os
import numpy as np
import pandas as pd
import scipy.stats
from scipy.stats import wilcoxon
from scipy.stats import ranksums
import re
import seaborn as sns, matplotlib.pyplot as plt
from bioinfokit.analys import stat
from statannotations.Annotator import Annotator

def get_options():
    parser = argparse.ArgumentParser(description="Analyses small scale chemical genomic screen data",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-p", "--PATH", help="Path to folder which contains IRIS files")
    parser.add_argument("-o", "--outputfile_prefix", help="Path and prefix for output file")
    parser.add_argument("-pf", "--PlateInfoPath", help="The path to the folder containing the plate info files.")
    parser.add_argument("-m", "--max_colony_size", help="Maximum colony size allowed, any colony larger than this will be removed",default=False,type=int)
    parser.add_argument("-wt", "--WildType", help="Name of wild type strain within plate info file.")
    parser.add_argument("-it", "--IRIS_type", help="Input IRIS morphology to test. Options: size,circularity,opacity", default="size")
    parser.add_argument("-col_plot", "--colourpalette", help="Name of Seaborn colour palette to use for the bar and swarm plots.", default="GnBu")
    parser.add_argument("-col_heat", "--colourheatmap", help="Name of Seaborn colour palette to use for the heatmap.", default="bwr_r")
    parser.add_argument("-wd", "--width", help="Figure width to use for the graphs.", default=5,type=int)
    parser.add_argument("-ht", "--height", help="Figure height to use for the graphs.", default=5,type=int)
    parser.add_argument("-r", "--rotation", help="X Axis label rotation", default=90,type=int)
    parser.add_argument("-cs", "--CircleSize", help="SwarmPlot circle size", default=2.5,type=float)
    parser.add_argument("-g", "--group", help="Group bar plots by strain or condition. Options = strain, condition.", default="condition")
    parser.add_argument("-pt", "--PlotType", help="Type of Plot. Options: barplot, swarmplot ", default="barplot")
    parser.add_argument("-rm","--remove_strain", help="txt file of strain names to remove separated by ';'. Names must match those in plate information file. E.g. mutant1;mutant2;mutant4",default=None)
    parser.add_argument("-ymax","--y_max", help="Maximum limit for y axis",default=None,type=float)
    parser.add_argument("-ymin","--y_min", help="Minimum limit for y axis",default=None,type=float)
    return parser.parse_args()


def ChemGAPP_Small():  
    options = get_options()
    PATH = options.PATH
    outputfile = options.outputfile_prefix
    PlateInfo = options.PlateInfoPath
    maximum_size = options.max_colony_size
    WT = options.WildType
    colour_palette = options.colourpalette
    colour_heatmap = options.colourheatmap
    IRIS = options.IRIS_type
    rote = options.rotation
    height1 = options.height
    width1 = options.width
    group1 = options.group
    circ_size = options.CircleSize
    plottype = options.PlotType
    removestrains = options.remove_strain
    ymaximum = options.y_max
    yminimum = options.y_min
    m = None
    indir = os.path.expanduser(PATH)
    # cycles through iris files and uses filename to produce column headers.
    for f in sorted(os.listdir(indir)):
        if f.endswith(".iris"): 
            g = pd.read_csv(os.path.join(indir, f),
                    comment='#',
                    index_col=[0, 1],
                    sep='\t')
            if m is None:
                m = g[IRIS]
                # replaces commas to periods, since these are not allowed within the filename and dashes to spaces
                m.name = (int(f.split('-')[2].split('_')[0]),
                          '-'.join(f.split('.')[0].split('-')[:2]).replace(",",".").replace("-"," "),
                          f.split('.')[0].split('_')[1])
                m = m.to_frame()
            else:
                m1 = g[IRIS]
                m1.name = (int(f.split('-')[2].split('_')[0]),
                          '-'.join(f.split('.')[0].split('-')[:2]).replace(",",".").replace("-"," "),
                          f.split('.')[0].split('_')[1])
                m1 = m1.to_frame()
                #sets files into a dataframe grouped by the condition and then columns as the A B C D or E etc
                m = m.join(m1, how='inner')
    # saves this initial dataset
    m.to_csv((outputfile+"_"+IRIS+"_Initial_dataset.csv"))
    m = m.apply(pd.to_numeric)
    m = m[sorted(m)]
    m_array = np.array(m)
    #calculates the size of the inputted plates
    mlen = len(m)
    if mlen == 1536:
        rlen = 32
        clen = 48
    if mlen == 384:
        rlen = 16
        clen = 24
    if mlen == 96:
        rlen = 8
        clen = 12
    n = m
    n = n.apply(pd.to_numeric)
    n = n[sorted(n)]
    m_ind = m.reset_index()
    # produces dataframe and array excluding the outer two rows and columns for each plate for plate middle mean (PMM) calculation
    mask = (m_ind['row'] != 1) & (m_ind['row'] != rlen) & (m_ind['column'] != 1) & (m_ind['column'] != clen) & (m_ind['row'] != 2) & (m_ind['row'] != (rlen-1)) & (m_ind['column'] != 2) & (m_ind['column'] != (clen-1))
    df_pmm = m_ind[mask]
    df_pmm = df_pmm.set_index(['row','column'])
    # produces dataframe and array of the outer two rows and columns for each plate
    df_outer = m_ind[-mask]
    df_outer = df_outer.set_index(['row','column'])
    pmm_array = np.array(df_pmm)
    #calculates median colony size for the entire dataset
    m_medain = np.nanmedian(m_array)
    
    n_array = np.array(n)
    ardf = np.zeros((mlen, 1))
    ardf.shape = (mlen,1)
    rounds = 0
    for c1,c2,i in zip(sorted(df_pmm.columns),sorted(df_outer.columns), range(len(m.columns))):
        rounds = rounds + 1
        #runs through each plate individually matching the plates for the outer and inner dataframes
        ar1 = pmm_array[:,i]
        ar2 = n_array[:,i]
        # finds the colony sizes within the 40th and 60th percentiles for PMM calculation
        pmm_40 = np.nanpercentile(ar1, 40)
        pmm_60 = np.nanpercentile(ar1, 60)
        mask2= (ar1 >= pmm_40) & (ar1 <= pmm_60) 
        pmm_perc_values = ar1[mask2]
        # finds mean of these 40th-60th percentile values = PMM
        PMM = pmm_perc_values.mean()
        #compares the distributions of outer colonies and inner colonies to check if first step of normalisation is required
        df1 = df_pmm.xs((c1), axis =1, drop_level=False)
        arA=np.array(df1)
        arA= np.array(arA).flatten()
        df2 = df_outer.xs((c2), axis =1, drop_level=False)
        arB=np.array(df2)
        arB= np.array(arB).flatten()
        w, p = ranksums(arA, arB)
        # if siginificantly different performs the first step and second step. 
        if p < 0.05:
            for ind, j in zip(m.index,range(len(ar2))):
                # for each colony within the outer two edges this calculates the median of the row and column in which they are located
                if ind[0] == 1 or ind[0] == rlen or ind[1] == 1 or ind[1] == clen or ind[0] == 2 or ind[0] == (rlen-1) or ind[1] == 2 or ind[1] == (clen-1):  
                    if ind[0] == 1:
                        p_median_list_1 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == 1]
                        p_median = np.nanmedian(p_median_list_1)
                    elif ind[0] == rlen:
                        p_median_list_32 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == rlen]
                        p_median = np.nanmedian(p_median_list_32)
                    elif ind[1] == 1:
                        p_median_list_1_1 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == 1]
                        p_median = np.nanmedian(p_median_list_1_1)
                    elif ind[1] == clen:
                        p_median_list_48 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == clen]
                        p_median = np.nanmedian(p_median_list_48)
                    elif ind[0] == 2:
                        p_median_list_2 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == 2]
                        p_median = np.nanmedian(p_median_list_2)
                    elif ind[0] == (rlen-1):
                        p_median_list_31 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == (rlen-1)]
                        p_median = np.nanmedian(p_median_list_31)
                    elif ind[1] == 2:
                        p_median_list_2_2 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == 2]
                        p_median = np.nanmedian(p_median_list_2_2)
                    elif ind[1] == (clen-1):
                        p_median_list_47 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == (clen-1)]
                        p_median = np.nanmedian(p_median_list_47)
                    # colonys within the outer two edges are then scaled such that the median of the row or column are equal to the PMM
                    # It then scales such that the the PMM is equal the median colony size of the entire datatset.
                    ar2[j] = (((ar2[j])*(PMM/p_median))*(m_medain/PMM))

                # If colonies are within the centre of the plate this scales such that the the PMM is equal
                # the median colony size of the entire datatset.
                elif ind[0] != 1 or ind[0] != rlen or ind[1] != 1 or ind[1] != clen or ind[0] != 2 or ind[0] != (rlen-1) or ind[1] != 2 or ind[1] != (clen-1):
                    ar2[j] = ((ar2[j])*(m_medain/PMM))

            ar2.shape = (mlen,1)
            #adds the column of adjusted scores to the 'normalised' dataframe 
            ardf = np.concatenate((ardf,ar2), axis=1)
        # if outer edge and inner colonies are not signficantly different 
        # then just scales entire plate such that the PMM is equal to the median colony size of entire dataset
        else:
            
            for ind, j in zip(m.index,range(len(ar2))):
                ar2[j] = ((ar2[j])*(m_medain/PMM))

            ar2.shape = (mlen,1)
            ardf = np.concatenate((ardf,ar2), axis=1)
    ardf = pd.DataFrame(ardf, index=m.index)
    #removes first column of zero values
    ardf = ardf.iloc[: , 1:]
    # adds in the multiheader columns
    ardf.columns = (pd.MultiIndex.from_tuples(sorted(m.columns)))
    ardf = ardf[sorted(ardf)]  
    ardf.to_csv(outputfile+"_"+IRIS+"_Normalised_dataset.csv")

    #renames the plate info file columns and adds row and column to index before sorting by index.
    def plate_info(file):
        p = pd.read_table(file)
        if len(p.columns) == 3:
            p.columns = ['row','column','strain']
            p = p.set_index(['row','column'])
            p = p.sort_index()
        else:
            print("Plate information file" + str(file) + "not in correct format.")
        return p
    # takes the specified directory for the plate information files
    directory = os.path.expanduser(PlateInfo)
    # iterate over plate files in that directory 
    files = []
    for filename in os.listdir(directory):
        f = os.path.join(directory, filename)
        # checking if it is a file
        if f.endswith(".txt"):
            if os.path.isfile(f):
                ##print(f)
                files.append(f)

    # looks for digits within the file names so that order of 
    # plate info files is sorted plate1, plate2, plate3 etc. not plate1, plate10, plate2 etc.
    def atoi(text):
        return int(text) if text.isdigit() else text

    def natural_keys(text):
        return [ atoi(c) for c in re.split(r'(\d+)', text) ]

    files.sort(key=natural_keys)
    #adds the plate files to a list
    plate_DFs = []
    for i in files:
        p = plate_info(i)
        plate_DFs.append(p)
    
    # makes set of all source plate numbers within dataset
    plates = {x[0] for x in ardf.columns}
    nm4 = ardf.copy(deep=False)
    df_with_strains = pd.DataFrame()
    # iterates through each source plate number and makes sub-dataset for each such that gene names can be appended.
    for a, n in zip(plate_DFs, sorted(plates)):
        df1 = (ardf.xs((n), axis=1, drop_level=True))
        df1.columns = [f'{i}_{j}' for i,j in df1.columns]
        #merges sub-dataset with the gene names, matching based on row and column location.
        df2 = pd.merge(df1,a,left_index=True, right_index=True)
        #adds merged dataset to new dataframe, such that all source plate named columns are within dataset.
        df_with_strains = pd.concat([df_with_strains , df2], ignore_index=True)
    df_with_strains = df_with_strains.rename(columns={'strain': 'Gene'})
    df_with_strains = df_with_strains.set_index('Gene')
    # splits columns back into multiheaders
    df_with_strains.columns = df_with_strains.columns.str.split('_', expand=True)
    df_with_strains = df_with_strains.sort_index()
    if maximum_size != False:
        df_with_strains[df_with_strains > float(maximum_size)] = np.nan
    df_with_strains= df_with_strains.reset_index()
    # removes rows named "EMPTY"
    df_with_strains = df_with_strains[df_with_strains['Gene'] != "EMPTY"]
    # if specified to remove certain strains will read the input and split by ";" to remove rows with specified names.
    if removestrains != None:
        with open(removestrains) as f:
            lines = f.readlines()
            for line in lines:
                inclstrains = str(line).split(";")
        for stra in inclstrains:
            print(stra)
            df_with_strains = df_with_strains[df_with_strains['Gene'] != stra]
    df_with_strains = df_with_strains.set_index(['Gene'])
    df_with_strains.to_csv(outputfile+"_"+IRIS+"_Final_dataset.csv")
    
    # For the bar plots:
    # averages the scores of rows with the same gene name.
    group = df_with_strains.groupby(level=0).mean()
    group2 = group.copy(deep=False)
    df3 = pd.DataFrame(index=group.index)
   
    # iterates through each condition
    for column in group:
        df1 = group[column]
        df1 = pd.DataFrame(df1)
        ar1 = np.array(df1)
        ar2 = np.array(df1)
        wt_mean = float(df1.loc[WT])
        # divides each averaged colony size by the mean wildtype colonysize for that condition
        for ind, j in zip(df1.index,range(len(ar1))):
            ar2[j] = (float(ar1[j])/wt_mean)
        #adds scored column to new dataframe
        df3 = np.concatenate((df3,ar2), axis=1)    
    df3 = pd.DataFrame(df3, index=group.index, columns=group.columns)
    df3.index.name = None
    # drops the WT row from the scored dataset.
    df3 = df3.drop(WT,axis=0)
    df3.to_csv(outputfile+"_"+IRIS+"_Scored_Dataset.csv")

    # if selected to group by condition, and bar plot then produces bar plot for each condition. 
    if group1 == "condition":
        if plottype == "barplot":
            conditions = {x[0] for x in df3.columns}
            for c in sorted(conditions):
                df1 = df3.xs((c), axis =1, drop_level=False)
                df2 = df1.melt(ignore_index=False)
                df2= df2.reset_index()
                df2.columns = ["Strain","Condition","Replicate","Score"]
                sns.set(style="white")
                sns.set_context("paper")
                fig, ax = plt.subplots()
                # plots bar plot and a swarm plot so the fitness scores for each replicates can be seen on the bar chart.
                ax = sns.barplot(x="Strain", y="Score",data=df2,capsize=.5,errwidth=0.8,palette=colour_palette,edgecolor="0.2")
                ax = sns.swarmplot(x="Strain", y="Score",color="0", data=df2, alpha=1,size=3)
                ax.set(xlabel= "",ylabel='Fitness Ratio')
                if ymaximum != None or yminimum != None:
                    ax.set_ylim(yminimum,ymaximum)
                plt.title(c)
                plt.xticks(rotation=rote)
                fig.set_size_inches(width1, height1)
                fig.set_dpi(1000)
                sns.despine()
                plt.savefig(outputfile1+"_"+c.replace(",",".")+"_"+IRIS+"_Bar_Plot.pdf", bbox_inches='tight')
                plt.clf()
        # if swarmplot selected produces swarmplots instead. Here the data is scored differently. 
        if plottype == "swarmplot":
            df_swarm1 = df_with_strains.reset_index()
            #makes df including just the wildtype values
            df_swarm2 = df_swarm1[df_swarm1['Gene'] == WT]
            # df_swarm3 = df_swarm1[df_swarm1['Gene'] != WT]
            # adds "0_" to WT column name such that it is always sorted first 
            # when sorting alphabetically, necessary for the ANOVA tests.
            df_swarm1['Gene'] = df_swarm1['Gene'].str.replace(WT,("0_"+WT))
            df_swarm3 = df_swarm1.sort_values("Gene") 
            # calculates the wildtype colony size mean for each condition plate.
            wt_mean = np.array(df_swarm2.mean())
            df_swarm3 = df_swarm3.set_index("Gene")
            df_swarm4 = pd.DataFrame(index=df_swarm3.index)
            # iterates through each condition plate and associated WT mean
            for column,mean in zip(df_swarm3,wt_mean):
                df1 = df_swarm3[column]
                df1 = pd.DataFrame(df1)
                ar1 = np.array(df1)
                ar2 = np.array(df1)
                #divides each individual colony size by the WT_mean, including for the WT values.
                for ind, j in zip(df1.index,range(len(ar1))):
                    ar2[j] = (float(ar1[j])/mean)
                #adds to a new dataframe
                df_swarm4 = np.concatenate((df_swarm4,ar2), axis=1)    
            df_swarm4 = pd.DataFrame(df_swarm4, index=df_swarm3.index, columns=df_swarm3.columns)
            df_swarm4.index.name = None
            conditions = {x[0] for x in df_swarm4.columns}
            #iterates through conditions and makes sub-dataset including all replicate plates for same condition.
            for c,i in zip(sorted(conditions),range(len(conditions))):
                df1 = df_swarm4.xs((c), axis =1, drop_level=False)
                #metls dataset so all scores are in sigular column
                df2 = df1.melt(ignore_index=False)
                df2= df2.reset_index()
                df2.columns = ["Strain","Condition","Replicate","Score"]
                # performs anova and tukey-hsd but only takes the values for comarison to the WT for each gene,
                # such that significance can be plotted on swarm plots
                res = stat()
                res.tukey_hsd(df=df2, res_var='Score', xfac_var='Strain', anova_model='Score ~ C(Strain)')
                stats1 = res.tukey_summary[res.tukey_summary['group1'] == ("0_"+WT)]
                pairs = list(zip(stats1.group2, stats1.group2))
                pvalues = list(stats1['p-value'])
                df2 = df2[df2.Strain != ("0_"+WT)]
                subcat_palette = sns.dark_palette("#8BF", reverse=True, n_colors=5)
                plotting_parameters = {
                    'data':    df2,
                    'x':       'Strain',
                    'y':       'Score'}
                sns.set(style="white")
                sns.set_context("paper")
                fig, ax = plt.subplots()
                ax = sns.swarmplot(x="Strain", y="Score",hue="Strain", data=df2, alpha=1,size=circ_size,palette=colour_palette)
                # adds mean bars
                ax = sns.boxplot(showmeans=True,
                        meanline=True,
                        meanprops={'color': 'k', 'ls': '-', 'lw': 1},
                        medianprops={'visible': False},
                        whiskerprops={'visible': False},
                        zorder=10,
                        x="Strain", y="Score",data=df2,
                        showfliers=False,
                        showbox=False,
                        showcaps=False,
                        ax=ax)
                #adds anova asterisk type annotations for sigificance. 
                annotator = Annotator(ax, pairs, **plotting_parameters)
                annotator.set_pvalues(pvalues)
                annotator.configure(line_width = 0)
                annotator.annotate()
                for patch in ax.artists:
                    r, g, b, a = patch.get_facecolor()
                    patch.set_facecolor((r, g, b, 0))    
                ax.set(xlabel= "Strain",ylabel='Fitness Ratio')
                if ymaximum != None or yminimum != None:
                    ax.set_ylim(yminimum,ymaximum)
                plt.title(c)
                plt.xticks(rotation=rote)
                fig.set_size_inches(width1, height1)
                fig.set_dpi(1000)
                sns.despine()
                plt.legend(loc='center left', bbox_to_anchor=(1, 0.5),
                              fancybox=True, shadow=False, ncol=1)
                plt.savefig(outputfile1+"_"+c+"_"+IRIS+"_Swarm_Plot.pdf", bbox_inches='tight')
                plt.clf()
    
    #here groups by strain and not condition.
    if group1 == "strain":
        if plottype == "barplot":
            #transposes dataset such that it can be grouped by strains.
            b = df3.T
            # makes set of all strains. 
            strain_list = {x for x in b.columns}
            #splits and iterates by strain.
            for c in sorted(strain_list):
                df1 = b.xs((c), axis=1, drop_level=False)
                df1 = df1.T
                #melts dataset such that all scores in one column with key being for the conditions
                df2 = df1.melt(ignore_index=False)
                df2.columns = ["Condition","Replicate","Score"]
                sns.set(style="white")
                sns.set_context("paper")
                fig, ax = plt.subplots()
                ax = sns.barplot(x="Condition", y="Score",data=df2,capsize=.5,errwidth=0.8,palette=colour_palette,edgecolor="0.2")
                ax = sns.swarmplot(x="Condition", y="Score",color="0", data=df2, alpha=1,size=3)
                ax.set(xlabel= "",ylabel='Fitness Ratio')
                if ymaximum != None or yminimum != None:
                    ax.set_ylim(yminimum,ymaximum)
                plt.xticks(rotation=rote)
                plt.title(c)
                fig.set_size_inches(width1, height1)
                fig.set_dpi(1000)
                sns.despine()
                plt.savefig(outputfile1+"_"+c+"_"+IRIS+"_Bar_Plot.pdf", bbox_inches='tight')
                plt.clf()
        if plottype == "swarmplot":
            #as above with condition grouped swarmplots, 
            # produces dataset with each individual colonzy size divided by WT mean for that condition. 
            # then calulates anova and tukey-hsd and takes p values for WT vs each strain. 
            df_swarm1 = df_with_strains.reset_index()
            df_swarm2 = df_swarm1[df_swarm1['Gene'] == WT]
            #df_swarm3 = df_swarm1[df_swarm1['Gene'] != WT]
            df_swarm1['Gene'] = df_swarm1['Gene'].str.replace(WT,("0_"+WT))
            df_swarm3 = df_swarm1.sort_values("Gene") 
            wt_mean = np.array(df_swarm2.mean())
            df_swarm3 = df_swarm3.set_index("Gene")
            df_swarm4 = pd.DataFrame(index=df_swarm3.index)
            for column,mean in zip(df_swarm3,wt_mean):
                df1 = df_swarm3[column]
                df1 = pd.DataFrame(df1)
                ar1 = np.array(df1)
                ar2 = np.array(df1)
                for ind, j in zip(df1.index,range(len(ar1))):
                    ar2[j] = (float(ar1[j])/mean)
                df_swarm4 = np.concatenate((df_swarm4,ar2), axis=1)    
            df_swarm4 = pd.DataFrame(df_swarm4, index=df_swarm3.index, columns=df_swarm3.columns)
            df_swarm4.index.name = None
            
            conditions = {x[0] for x in df_swarm4.columns}
            genes = {x for x in df_swarm4.index}
            leng = len(genes)-1
            # produces zeros matrix that can be filled for [(gene1,gene1),condition,p-value], 
            # need to compare pairs to themselves since plots sorted by strain then plot based on the conditions split by gene name
            anovas = np.zeros((leng, 3),dtype=object)
            anovas.shape = (leng,3)
            for c in sorted(conditions):
                df1 = df_swarm4.xs((c), axis =1, drop_level=False)
                df2 = df1.melt(ignore_index=False)
                df2 = df2.reset_index()
                df2.columns = ["Strain","Condition","Replicate","Score"]
                res = stat()
                res.tukey_hsd(df=df2, res_var='Score', xfac_var='Strain', anova_model='Score ~ C(Strain)')
                #takes only those compared to the WT
                stats1 = res.tukey_summary[res.tukey_summary['group1'] == ("0_"+WT)]
                # makes list of the current condition same length as number of genes - 1. 
                # Allows for zipping to the pairs and p-values for the matrix.
                cond = ([c] * leng)
                pairs = list(zip(stats1.group2, stats1.group2))
                pvalues = list(stats1['p-value'])
                grouped = list(zip(pairs,cond,pvalues))
                grouped = np.array(grouped,dtype=object)
                anovas = np.append(anovas,grouped, axis=0)
            anovas = anovas[leng:len(anovas)]
            df_swarm4 = df_swarm4[df_swarm4.index != ("0_"+WT)]
            #transposes dataset such that can be split by strain instead of condition.
            b = df_swarm4.T
            conditions = {x for x in b.columns}
            for c,i in zip(sorted(conditions),range(len(conditions))):
                df1 = b.xs((c), axis=1, drop_level=False)
                df1 = df1.T
                df2 = df1.melt(ignore_index=False)
                df2.columns = ["Condition","Replicate","Score"]
                #finds the annotation pair (condition,condition) and pvalue for the current strain for annotation of significance scores.
                m = [row for row in anovas if c == row[0][0]]
                pair1 = [(row[1],row[1]) for row in m]
                pvalue1 = [row[2] for row in m]
                df2 = df2[df2.Condition != "WT"]
                subcat_palette = sns.dark_palette("#8BF", reverse=True, n_colors=5)
                plotting_parameters = {
                'data':    df2,
                'x':       'Condition',
                'y':       'Score',
                'palette': subcat_palette[1:]}
                sns.set(style="white")
                sns.set_context("paper")
                fig, ax = plt.subplots()
                ax = sns.swarmplot(x="Condition", y="Score",hue="Condition", data=df2, alpha=1,size=circ_size,palette=colour_palette)
                ax = sns.boxplot(showmeans=True,
                        meanline=True,
                        meanprops={'color': 'k', 'ls': '-', 'lw': 1},
                        medianprops={'visible': False},
                        whiskerprops={'visible': False},
                        zorder=10,
                        x="Condition", y="Score",data=df2,
                        showfliers=False,
                        showbox=False,
                        showcaps=False,
                        ax=ax)
                # adds p-value annotations for each condition.
                annotator = Annotator(ax, pair1, **plotting_parameters)
                annotator.set_pvalues(pvalue1)
                annotator.configure(line_width = 0)
                annotator.annotate()
                for patch in ax.artists:
                    r, g, b, a = patch.get_facecolor()
                    patch.set_facecolor((r, g, b, 0))    
                ax.set(xlabel= "Condition",ylabel='Fitness Ratio')
                if ymaximum != None or yminimum != None:
                    ax.set_ylim(yminimum,ymaximum)
                plt.title(c)
                plt.xticks(rotation=rote)
                fig.set_size_inches(width1, height1)
                fig.set_dpi(1000)
                sns.despine()
                plt.legend(loc='center left', bbox_to_anchor=(1, 0.5),
                              fancybox=True, shadow=False, ncol=1)
                plt.savefig(outputfile1+"_"+c+"_"+IRIS+"_Swarm_Plot.pdf", bbox_inches='tight')
                plt.clf()
    #produces heatmap based on the same dataset used for the barplots
    # transposes data such that rows are conditions
    b = df3.T
    # averages replicate scores
    c = b.groupby(level=0).mean()
    #transposes back for plotting of the heatmap
    x = c.T
    for i in range(len(x.columns)):
        x = x.rename(columns={x.columns[i]: x.columns[i]})
    c = x.T
    sns.set(style="white")
    sns.set_context("paper")
    colp = sns.color_palette(colour_heatmap, as_cmap=True)
    fig, ax = plt.subplots()
    ax = sns.heatmap(c,center=1,cmap=colp,square=False,annot=True,annot_kws={"size": 4},fmt=".4f",linewidths=.1,linecolor='0',vmin=0, vmax=2)
    fig.set_size_inches(width1, height1)
    plt.savefig(outputfile1+"_"+IRIS+"_Heatmap.pdf", bbox_inches='tight')
    plt.clf()
    return df3

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def get_options():

    parser = argparse.ArgumentParser(description="Produces a bar plot showing the counts of conditions with a certain number of plates lost at different thresholds of Variance and Mann-Whitney mean p-value variance.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The input file for the module. Uses the output files from Pass_Fail_Conditions.py, either Variance or Mann_Whitney.")
    parser.add_argument("-o", "--OutputPlot", help="Name of output file, a PDF of the bar chart")
    return parser.parse_args()

def main():
    options = get_options()
    pass_fail_file = options.InputFile
    otptplot = options.OutputPlot
    
    #takes the pass fail file from Pass_Fail_Conditions.py, either Variance or Mann_Whitney.
    Pass_Fail = pd.read_csv(pass_fail_file)
    Pass_Fail = Pass_Fail.set_index(['Condition','Batch'])
    bar = pd.DataFrame(columns = ['Condition','Batch', 'Threshold','Percentage'])
    #iterates over rows by the condition and batch
    for p5 in sorted(Pass_Fail.index):
        dfpf14 = Pass_Fail.xs((p5), axis=0, drop_level=False)
        dfpf14 = pd.DataFrame(dfpf14)
        #transposes the data so number of F's can be counted to produce percentage fails by dividing by overall count and multiplying by 100.
        dfpf14 = pd.DataFrame.transpose(dfpf14)
        for cols in dfpf14.columns:
            if cols[0] == 'M' or cols[0] == 'N' or cols[0] == 'A':
                failed = ((len(dfpf14[dfpf14[cols] == 'F'])/dfpf14[cols].count())*100)
                name=(dfpf14.index[0][0],dfpf14.index[0][1],cols,failed)
                columns = list(bar)
                data = []
                zipped = zip(columns, name)
                a_dictionary = dict(zipped)
                data.append(a_dictionary)
                bar = bar.append(data, True) 
        bar = bar.round(2)
    bar_list = bar['Percentage'].astype(float).values
    b3 = {x for x in bar_list}
    b3 = list(sorted(b3))
    bar['Percentage'] = pd.Categorical(bar['Percentage'], categories=b3,ordered=True)
    sns.set_theme(style="whitegrid")
    b = sns.displot(bar, x='Threshold', hue='Percentage',
                    multiple='stack', legend = False, palette=("White","orange"))
    b.set_axis_labels("", "Count of Conditions Lost")
    b.set_xticklabels(rotation=90)
    plt.title("Count of Conditions lost at Different Thresholds")
    plt.savefig(otptplot, bbox_inches='tight')
    return bar

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import scipy.stats as stats

def get_options():
    parser = argparse.ArgumentParser(description=" Compares each replicate colony to find outliers within colony size for each plate.",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The normalised csv file from Check_Normalisation.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file of the dataset where colony sizes are replaced with the colony type values.")
    return parser.parse_args()

def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    def z_score_method(df):
        #performs z-score test and between replicates of same mutant in 
        # same condition plate and takes the absolute value of the z-score value
        z = np.abs(stats.zscore(df))
        threshold = 1.4
        outlier = []
        #goes through each z-score value and position  
        for i, v in enumerate(z):
            #if nan then continue
            if np.any(v) == 'nan':
                continue
            else:
                #if the z-score value is greater than the threshold then 
                # it is an outlier and the postion is appended to 'outlier' list
                if v > threshold:
                    outlier.append(i)
                else:
                    continue
            return outlier
    m = pd.read_csv(inputfile,
                          index_col=[0, 1],
                          header=[0, 1, 2, 3])
    conditions = {x[0:2] for x in m.columns}
    rounds = 0
    ABSN = pd.DataFrame(index=m.index)
    for c in sorted(conditions):
        rounds = rounds + 1
        print(rounds)
        df1 = m.xs((c), axis =1, drop_level=False)
        ar1=np.array(df1)
        #makes array that matches the colony size data, however instead of values is all N's for Normal
        ar2=np.full(np.shape(ar1), "N")
        #iterates through row of the columns array.
        for j in range(len(ar1)):
            #iterates through the position and value of row     
            for l, vs in enumerate(ar1[j]):
                #if the value is nan, the matching position in the N array is changed to an X
                if str(vs) == "nan":
                    ar2[j][l] = "X" 
            #if all the replicates in the row are 0 then the row is skipped 
            if np.all(ar1[j]) == 0:
                continue
            else: 
                #performs the z-score method for the replicates and makes list of the outliers
                outlier = z_score_method(ar1[j])
                #if there are outliers then it will find the mean of the replicates and then look 
                # at the positions specified as outliers and see if they are larger than the mean
                #  or smaller and designated an B or S accordingly in the N's array.
                if outlier != None:
                    if len(outlier) > 0:
                        mean = np.nanmean(ar1[j])
                        if ar1[j][outlier] < mean:
                            ar2[j][outlier] ="S"
                        else:
                            ar2[j][outlier]="B"
        #the array of Ns, Bs, Ss and Xs are formatted and saved as a df.
        ar_df = pd.DataFrame(ar2, index=m.index)
        ABSN = pd.concat([ABSN, ar_df], axis=1)
    ABSN.columns = (pd.MultiIndex.from_tuples(sorted(m.columns)))
    ABSN.to_csv(outputfile)
    return ABSN

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np

def get_options():
    parser = argparse.ArgumentParser(description="Counts the number of each colony type within each plate and the percentage of each colony type.",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The CSV output file from Z_Score.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file with the counts and percentages of the each colony type.")
    return parser.parse_args()
    

def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    #reads the csv file of the datasets of Ns,Bs,Ss and Xs from Z_score.py
    ABSN = pd.read_csv(inputfile,
                      index_col=[0, 1],
                      header=[0, 1, 2, 3])
    print("File Uploaded")
    #create empty dataframe for data to append into.
    zero_count = pd.DataFrame(columns=['Plate','Condition','Replicate','Batch','Normal','Bigger','Smaller',
                                   'NaN','% Normal','% Bigger','% Smaller','% NaN'])
    replicate = {x[0:4] for x in ABSN.columns}
    rounds = 0
    #iterates through every column of the dataset and counts the N, B, S 
    # and X values then calculates the percentage of the plate they represent
    #these are then appended into the empty dataset for each plate condition replicate and batch. 
    for r in sorted(replicate):
        rounds = rounds + 1
        print(rounds,r)
        df1 = ABSN.xs((r), axis =1, drop_level=False)
        ar1 = np.array(df1)
        count_S = int(np.count_nonzero(ar1 == "S", axis=0))
        count_B = int(np.count_nonzero(ar1 == "B", axis=0))
        count_N = int(np.count_nonzero(ar1 == "N", axis=0))
        count_nan = int(np.count_nonzero(ar1 == "X", axis=0))
        total = (count_S+count_N+count_B+count_nan)
        B_perc = (count_B/total)*100
        S_perc = (count_S/total)*100
        N_perc = (count_N/total)*100
        Nan_perc = (count_nan/total)*100
        name = (df1.columns[0][0],df1.columns[0][1],
                    df1.columns[0][2],df1.columns[0][3],
                    count_N, count_B, count_S, count_nan,N_perc,B_perc,
                    S_perc,Nan_perc)
        columns = list(zero_count)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        zero_count = zero_count.append(data, True)
        zero_count.to_csv(outputfile, index=False)
    return zero_count

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np

def get_options():
    parser = argparse.ArgumentParser(description="Computes the S-scores from the normalised dataset. ",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The normalised csv file from Check_Normalisation.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file of the dataset as S-scores")
    parser.add_argument("-s", "--Scale", help="Scale scores to and inter-quartile range of 1.35. Options: True, False", default=True)
    return parser.parse_args()
    
    
    
def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    scale = options.Scale
    nm = pd.read_csv(inputfile,
                      index_col=[0, 1],
                      header=[0, 1, 2, 3])
    nm = nm.reindex(sorted(nm.columns), axis=1)
    #replaces inf values with nan
    nm = nm.replace(np.inf, np.nan)
    nm_array = np.array(nm)
    nmlen = len(nm)

    plates = {x[0:2] for x in nm.columns}
    repnumber = np.array([])
    #splits by condition and source plate and counts number of replicates then finds median number of replicates
    for p in sorted(plates):
        df1 = (nm.xs((p), axis =1, drop_level=False))
        ar1 = np.array(df1)
        length = ar1.shape[1]
        repnumber = np.append(repnumber,length)
    ncont = np.nanmedian(repnumber) 
    print("ncont calculated")  
    

    plates2 = {x[0] for x in nm.columns}
    varcontfull = []
    #splits dataset by source plate and iterates through
    for p in sorted(plates2):
        varcontdf = np.zeros((nmlen, 1))
        varcontdf.shape = (nmlen,1)
        #produces df with all conditions for a specific source plate.
        df1 = (nm.xs((p), axis =1, drop_level=False))
        #swaps levels so batch included in splitting for attributes
        df1 = df1.swaplevel(0,1,axis=1)
        df1 = df1.swaplevel(1,3,axis=1)
        #splits by condition and batch
        conds = {x[0:2] for x in df1.columns}
        for p2 in sorted(conds):
            df2 = (df1.xs((p2), axis =1, drop_level=False))
            ar1 = np.array(df2)
            #calculates variance for each row as an array
            varvallist = np.nanvar(ar1,axis=1)
            varvallist = np.array(varvallist)
            varvallist.shape = (nmlen,1)
            #produces df with variance for every mutant in every condition plate
            varcontdf = np.concatenate((varcontdf, varvallist),axis=1)
        varcontdf = pd.DataFrame(varcontdf)
        varcontdf = varcontdf.iloc[: , 1:] 
        ar2 = np.array(varcontdf)
        #calculates the mean variance for each row to make a list of vcont values.
        varcontlist = np.nanmedian(varcontdf,axis=1)
        varcontfull.append(varcontlist)
    print("vcont calculated")
        
    ucontfull = []
    #splits by plate so only same mutants in each row
    for p in sorted(plates2):
        ucontlist = []
        #calculates the median colony size for all rows across all conditions to make list of ucont values
        df1 = (nm.xs((p), axis =1, drop_level=False))
        ar1 = np.array(df1)
        uvals = np.nanmedian(ar1,axis=1)
        ucontlist = np.append(ucontlist,uvals)
        ucontfull.append(ucontlist)
    print("ucont calculated")

    
    
    ################################################
    
    
    
    #calculates median relative error for all mutants in all conditions
    medrelerror = (np.nanstd(nm_array,dtype=np.float64)/(np.nanmean(nm_array,dtype=np.float64)))
    rounds = 0
    final_s_score = pd.DataFrame(index=nm.index)
    #splits by condition and source plate
    for p in sorted(plates):
        #produce empty array for S-scores
        s_score_array = np.array([])
        rounds = rounds + 1
        print(rounds, p)
        df1 = (nm.xs((p), axis =1, drop_level=False))
        ar1 = np.array(df1)
        #produce minimum for varexp, mean standard deviation squared.
        varmin = np.nanmean(np.nanstd(ar1,axis=1))
        varmin2 = varmin*varmin
        # runs through the rows of the array zipped to the ucont and varcont list values associated to that row.
        for r,k,vc in zip(range(len(ar1)), ucontfull[int(p[0])-1],varcontfull[int(p[0])-1]):
            #calculate var exp by getting the std squared of the replicate colonysizes
            varexpstd = np.nanstd(ar1[r])
            varexp = varexpstd*varexpstd
            #if varexp lower than the minimium bound then varexp becomes the minimum bound
            if varexp < varmin2:
                varexp = varmin2
            #nexp is the number of replicates for the plate and condition. thus takes the shape value as this matches this.
            nexp = (ar1.shape[1])
            ncont = np.nanmedian(repnumber)
            # calculates uexp by finding average of the replciates colony sizes for mutant of interest
            uexp = np.nanmean(ar1[r])
            ucont = k
            #minimum bound for varcont is ucont multiplied by the median relative error
            minboundvarcont = ucont*medrelerror 
            #thus if vc larger then varcont is vc else the minimum bound is taken
            if vc > minboundvarcont:
                varcont = vc
            else:
                varcont = minboundvarcont
            #calculates svar and s-scores using the variables for each mutant of interest in the condition of interest
            svar = (varexp*(nexp-1)+varcont*(ncont-1))/(nexp+ncont-2)
            S_score = (uexp-ucont)/(np.sqrt(svar/nexp+svar/ncont))
            #appends s-score to an array to build up each row of each column to concatenate into a full dataset
            s_score_array = np.append(s_score_array,S_score)  
        ar_df = pd.DataFrame(s_score_array, index=nm.index)
        final_s_score = pd.concat([final_s_score, ar_df], axis=1)
    #adds columns back in for just the conditions and source plate numbers
    final_s_score.columns = (pd.MultiIndex.from_tuples(sorted(nm.columns.droplevel([2,3]).unique())))
    final_s_score = final_s_score.replace([np.inf, -np.inf], np.nan)
    if scale == True:
        mlen = len(final_s_score)
        ardf = np.zeros((mlen, 1))
        ardf.shape = (mlen,1)
        #scales everything such that the interquartile range of each column is equal to 1.35
        for c in sorted(final_s_score.columns):
            df1 = pd.DataFrame(final_s_score.xs(c,axis=1,drop_level=False))
            ar1 = np.array(df1)
            ar2 = np.array(df1)
            mean = np.nanmean(ar1)
            iqr = (np.nanpercentile(ar1,[75])-np.nanpercentile(ar1,[25]))
            for i in range(len(ar1)):
                ar2[i] = (ar1[i]*(1.35/iqr))
            ar2.shape = (mlen,1)
            ardf = np.concatenate((ardf,ar2), axis=1)
        ardf = pd.DataFrame(ardf, index=final_s_score.index)
        ardf = ardf.iloc[: , 1:]
        ardf.columns = (pd.MultiIndex.from_tuples(sorted(final_s_score.columns)))
        ardf = ardf[sorted(ardf)]
        ardf.to_csv(outputfile)
    if scale == "False":
        ardf = final_s_score
        ardf.to_csv(outputfile)
    return ardf

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
from pickle import FALSE
import pandas as pd
import numpy as np
import scipy
import scipy.stats
from scipy.stats import wilcoxon
from scipy.stats import ranksums

def get_options():
    parser = argparse.ArgumentParser(description="Checks each plate individually to see if outer-edge normalisation is required due to plate effects.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The CSV file from Iris_to_dataset.py of the colony dataset with conditions across the top and row/column coordinates downwards")
    parser.add_argument("-o", "--OutputFile", help="CSV file of the normalised colony sizes.")
    parser.add_argument("-m", "--max_colony_size", help="Maximum colony size allowed, any colony larger than this will be set to this maximum",default=False,type=int)
    return parser.parse_args()

def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    maximum_size = options.max_colony_size
    m = pd.read_csv(inputfile,index_col=[0, 1],header=[0, 1, 2, 3])
    m = m.apply(pd.to_numeric)
    m = m[sorted(m)]
    m_array = np.array(m)
    #calculates the size of the inputted plates
    mlen = len(m)
    if mlen == 1536:
        rlen = 32
        clen = 48
    if mlen == 384:
        rlen = 16
        clen = 24
    if mlen == 96:
        rlen = 8
        clen = 12
    n = pd.read_csv(inputfile,index_col=[0, 1],header=[0, 1, 2, 3])
    n = n.apply(pd.to_numeric)
    n = n[sorted(n)]
    m_ind = m.reset_index()
    # produces dataframe and array excluding the outer two rows and columns for each plate for plate middle mean (PMM) calculation
    mask = (m_ind['row'] != 1) & (m_ind['row'] != rlen) & (m_ind['column'] != 1) & (m_ind['column'] != clen) & (m_ind['row'] != 2) & (m_ind['row'] != (rlen-1)) & (m_ind['column'] != 2) & (m_ind['column'] != (clen-1))
    df_pmm = m_ind[mask]
    df_pmm = df_pmm.set_index(['row','column'])
    # produces dataframe and array of the outer two rows and columns for each plate
    df_outer = m_ind[-mask]
    df_outer = df_outer.set_index(['row','column'])
    pmm_array = np.array(df_pmm)
    #calculates median colony size for the entire dataset
    m_medain = np.nanmedian(m_array)
    
    #Change all false zeros to NaN in the dataset
    conditions = {x[0:2] for x in m.columns}
    n2 = np.zeros((mlen, 0))
    n2.shape = (mlen,0)
    for c in sorted(conditions):
        df1 = n.xs((c), axis =1, drop_level=False)
        ar1=np.array(df1)
        ar1 = ar1.astype(np.float64)
        for j in range(len(ar1)):
            if np.all(ar1[j]) == 0:
                if not np.any(ar1[j]) == 0:
                    for l, vs in enumerate(ar1[j]):
                        if vs == 0:
                            ar1[j][l] = "NaN"
        n2 = np.concatenate((n2,ar1),axis=1)
    n2 = pd.DataFrame(n2, index=m.index)
    n2.columns = (pd.MultiIndex.from_tuples(sorted(n.columns)))
    n2 = n2[sorted(n2)]
    n_array = np.array(n2)
    
    ardf = np.zeros((mlen, 1))
    ardf.shape = (mlen,1)
    rounds = 0
    #runs through each plate individually matching the plates for the outer and inner dataframes
    for c1,c2,i in zip(sorted(df_pmm.columns),sorted(df_outer.columns), range(len(m.columns))):
        rounds = rounds + 1
        print(rounds)
        ar1 = pmm_array[:,i]
        ar2 = n_array[:,i]
        # finds the colony sizes within the 40th and 60th percentiles for PMM calculation
        pmm_40 = np.nanpercentile(ar1, 40)
        pmm_60 = np.nanpercentile(ar1, 60)
        mask2= (ar1 >= pmm_40) & (ar1 <= pmm_60) 
        pmm_perc_values = ar1[mask2]
        # finds mean of these 40th-60th percentile values = PMM
        PMM = pmm_perc_values.mean()
        #compares the distributions of outer colonies and inner colonies to check if first step of normalisation is required
        df1 = df_pmm.xs((c1), axis =1, drop_level=False)
        arA=np.array(df1)
        arA= np.array(arA).flatten()
        df2 = df_outer.xs((c2), axis =1, drop_level=False)
        arB=np.array(df2)
        arB= np.array(arB).flatten()
        w, p = ranksums(arA, arB)
        # if siginificantly different performs the first step and second step. 
        if p < 0.05:
            print("Different Dist")
            for ind, j in zip(m.index,range(len(ar2))):
                # for each colony within the outer two edges this calculates the median of the row and column in which they are located
                if ind[0] == 1 or ind[0] == rlen or ind[1] == 1 or ind[1] == clen or ind[0] == 2 or ind[0] == (rlen-1) or ind[1] == 2 or ind[1] == (clen-1):  
                    if ind[0] == 1:
                        p_median_list_1 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == 1]
                        p_median = np.nanmedian(p_median_list_1)
                    elif ind[0] == rlen:
                        p_median_list_32 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == rlen]
                        p_median = np.nanmedian(p_median_list_32)
                    elif ind[1] == 1:
                        p_median_list_1_1 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == 1]
                        p_median = np.nanmedian(p_median_list_1_1)
                    elif ind[1] == clen:
                        p_median_list_48 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == clen]
                        p_median = np.nanmedian(p_median_list_48)
                    elif ind[0] == 2:
                        p_median_list_2 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == 2]
                        p_median = np.nanmedian(p_median_list_2)
                    elif ind[0] == (rlen-1):
                        p_median_list_31 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[0] == (rlen-1)]
                        p_median = np.nanmedian(p_median_list_31)
                    elif ind[1] == 2:
                        p_median_list_2_2 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == 2]
                        p_median = np.nanmedian(p_median_list_2_2)
                    elif ind[1] == (clen-1):
                        p_median_list_47 = [list(m_array[c:c+1,i]) for c, ind in zip(range(len(m_array)),m.index) if ind[1] == (clen-1)]
                        p_median = np.nanmedian(p_median_list_47)
                    # colonys within the outer two edges are then scaled such that the median of the row or column are equal to the PMM
                    # It then scales such that the the PMM is equal the median colony size of the entire datatset.
                    ar2[j] = (((ar2[j])*(PMM/p_median))*(m_medain/PMM))

                    if maximum_size != False:
                        if ar2[j] > maximum_size:
                            ar2[j] = maximum_size
                # If colonies are within the centre of the plate this scales such that the the PMM is equal
                # the median colony size of the entire datatset.
                elif ind[0] != 1 or ind[0] != rlen or ind[1] != 1 or ind[1] != clen or ind[0] != 2 or ind[0] != (rlen-1) or ind[1] != 2 or ind[1] != (clen-1):
                    ar2[j] = ((ar2[j])*(m_medain/PMM))
                    if maximum_size != False:
                        if ar2[j] > maximum_size:
                            ar2[j] = maximum_size
            ar2.shape = (mlen,1)
            ardf = np.concatenate((ardf,ar2), axis=1)
        # if outer edge and inner colonies are not signficantly different 
        # then just scales entire plate such that the PMM is equal to the median colony size of entire dataset
        else:
            print(c1,"Same Dist")
            for ind, j in zip(m.index,range(len(ar2))):
                ar2[j] = ((ar2[j])*(m_medain/PMM))
                if maximum_size != False:
                    if ar2[j] > maximum_size:
                        ar2[j] = maximum_size
            ar2.shape = (mlen,1)
            ardf = np.concatenate((ardf,ar2), axis=1)
    ardf = pd.DataFrame(ardf, index=m.index)
    ardf = ardf.iloc[: , 1:]
    ardf.columns = (pd.MultiIndex.from_tuples(sorted(m.columns)))
    ardf = ardf[sorted(ardf)] 
    ardf.to_csv(outputfile)
    return ardf
    
if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import numpy as np
import pandas as pd
import re
import os
def get_options():
    parser = argparse.ArgumentParser(description="Produces datasets with genetic interaction scores for genetic interaction screens",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--inputfiles", help="Path to IRIS files.")
    parser.add_argument("-p", "--PATH", help="Path for the output files.")
    parser.add_argument("-n", "--nameinfofiles", help="Path to plate information files. Plate info files should be txt files, with the columns: Row, Column, Strain, Replicate, Order, Set.")
    return parser.parse_args()

def GI_dataset():    
    options = get_options()
    indir = options.inputfiles
    PATH = options.PATH
    infodir = options.nameinfofiles
    indir = os.path.expanduser(indir)                     
    m = None
    # cycles through iris files and uses filename to produce column headers.
    for f in os.listdir(indir):
        if f.endswith(".iris"):
            g = pd.read_csv(os.path.join(indir, f),
                    comment='#',
                    index_col=[0, 1],
                    sep='\t')
            if m is None:
                try:
                    m = g['colony size']
                except:
                    m = g['size']
                    m.name = (f.split('_')[0],f.split('.')[0].split('_')[1])
                    m = m.to_frame()
            else:
                try:
                    m1 = g['colony size']
                except:
                    m1 = g['size']
                    m1.name = (f.split('_')[0],
                            f.split('.')[0].split('_')[1])
                    m1 = m1.to_frame()
                #sets them in a dataframe grouped by the secondary gene name and then replicates.
                m = m.join(m1, how='inner')
    
    # renames the plate info file columns and adds row and column to index before sorting by index.
    def plate_info(file):
        p = pd.read_table(file)
        if len(p.columns) != 6:
            print("ERROR: Info file formatting incorrect! Should be 6 columns: Row,Column,Strain,Replicate,Order,Set")
        if len(p.columns) == 6:
            p.columns = ['row','column','strain','Replicate','Order','Set']
            p = p.set_index(['row','column'])
            p = p.sort_index()
        return p


    # looks for digits within the file names so that order of 
    # plate info files is sorted plate1, plate2, plate3 etc. not plate1, plate10, plate2 etc.        
    def atoi(text):
        return int(text) if text.isdigit() else text

    def natural_keys(text):
        return [ atoi(c) for c in re.split(r'(\d+)', text) ]

    # iterate over plate files in that directory           
    infodir = os.path.expanduser(infodir)
    files = []
    for filename in os.listdir(infodir):
        f = os.path.join(infodir, filename)
        # checking if it is a file
        if f.endswith(".txt"):
            if os.path.isfile(f):
                files.append(f)
    files.sort(key=natural_keys)
    
    #adds the plate files to a list
    plate_DFs = []
    for i in files:
        p = plate_info(i)
        plate_DFs.append(p)
    plates = {x[0:2] for x in m.columns}
    m2 = pd.DataFrame(m.copy(deep=False))
    # splits by secondary gene name 
    columns1 = [x[0] for x in m2.columns]
    df_with_strains = pd.DataFrame(columns = [columns1[0],'strain','Replicate','Order','Set'])
    # adds the gene names, replicate number, set and order to the colony size data based on row and column
    for a, n in zip(plate_DFs, sorted(plates)):
        df1 = (m.xs((n), axis =1, drop_level=True))
        df1 = pd.DataFrame(df1)
        df2 = pd.merge(df1,a,left_index=True, right_index=True)
        df2.columns = [n[0],'strain','Replicate','Order','Set']
        df_with_strains = pd.concat([df_with_strains, df2], ignore_index=True)
    df_with_strains = df_with_strains.rename(columns={'strain': 'Gene'})
    df2 = df_with_strains.set_index(['Set','Replicate','Order','Gene'])
    set = {x[0] for x in df2.index}
    #splits by set, thus different gene sets have their own output files. 
    for s in sorted(set):
        df = pd.DataFrame()
        df1 = df2.xs((s), axis=0, drop_level=True)
        replicate = {x[0] for x in df1.index}
        for r in sorted(replicate):
            dfb = df1.xs((r), axis=0, drop_level=False)
            dfb = dfb.sort_index(level="Order")
            #calculates the fitness ratios and double expected ratios and adds them to the dataset
            name = (dfb.iloc[1][0]/dfb.iloc[0][0],dfb.iloc[2][0]/dfb.iloc[0][0],(dfb.iloc[1][0]/dfb.iloc[0][0])*(dfb.iloc[2][0]/dfb.iloc[0][0]),dfb.iloc[3][0]/dfb.iloc[0][0])
            columns = list([dfb.iloc[1].name[2],"Secondary Gene","Double Expected","Double Observed"])
            data = []
            zipped = zip(columns, name)
            a_dictionary = dict(zipped)
            data.append(a_dictionary)
            df = df.append(data, True)
        df1.to_csv((PATH+"/"+dfb.iloc[2].name[2]+"_Colony_sizes.csv"))
        df.to_csv((PATH+"/"+dfb.iloc[2].name[2]+"_Interaction_Scores.csv"))

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import os
import sys
import numpy as np
import pandas as pd

def get_options():
    parser = argparse.ArgumentParser(description="Takes folder of Iris files and produces Dataset for ChemGAPP",
                                 formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-p", "--PATH", help="Path to folder which contains IRIS files, IRIS file names should be in the format: CONDITION-concentration-platenumber-batchnumber_replicate.JPG.iris")
    parser.add_argument("-o", "--outputfile", help="Name of output file, should be a .csv file")
    parser.add_argument("-it", "--IRISphenotype", help="IRIS phenotype from IRIS files you wish to analyse",default='size')
    return parser.parse_args()


def main():    
    options = get_options()
    PATH = options.PATH
    outputfile = options.outputfile
    phen = options.IRISphenotype
    m = None
    # cycles through iris files and uses filename to produce column headers.
    indir = os.path.expanduser(PATH)
    for f in sorted(os.listdir(indir)):
        if f.endswith(".iris"): 
            sys.stderr.write(f + '\n')
            print(os.path.join(indir, f))
            g = pd.read_csv(os.path.join(indir, f),
                    comment='#',
                    index_col=[0, 1],
                    sep='\t')
            if m is None:
                try:
                    m = g[phen]
                except:
                    m = g[phen]
                m.name = (int(f.split('-')[2].split('_')[0]),
                          '-'.join(f.split('.')[0].split('-')[:2]).replace(",","."),
                          f.split('.')[0].split('_')[1],"Batch"+ f.split('-')[3].split('_')[0]
                          )
                m = m.to_frame()
            else:
                try:
                    m1 = g[phen]
                except:
                    m1 = g[phen]
                m1.name = (int(f.split('-')[2].split('_')[0]),
                          '-'.join(f.split('.')[0].split('-')[:2]).replace(",","."),
                          f.split('.')[0].split('_')[1],
                          "Batch"+ f.split('-')[3].split('_')[0])
                m1 = m1.to_frame()
            
                m = m.join(m1, how='inner')
    m.to_csv(outputfile)
    
if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np

def get_options():
    parser = argparse.ArgumentParser(description="The variance of the replicate means for each condition is calculated and then the average of these variance is calculated for each plate within that conditions, i.e the variance between replicate plate A,B,C,D for plate 1 of condition A, and then the average of plate 1, 2, 3 etc. for condition A."
                                     ,formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The CSV file with the mean u statistics and p values for each replicate from Mann_Whitney_Plate_Level.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file with the mean variance values for the u statistic and p values of the mann-whitney test for each condition.")
    return parser.parse_args()

def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    #reads the CSV file with the mean u statistics and p values for each replicate from Mann_Whitney_Plate_Level.py
    Pmean = pd.read_csv(inputfile)
    Pmean = Pmean.set_index(['Condition','Plate','Batch','Replicate'])
    cond2 = {x[0:3] for x in Pmean.index}
    P_var = pd.DataFrame(columns=['Plate','Condition','Batch','Variance U-Stat','Variance P-Value'])
    #iterates through dataset splitting by plate, condition and batch number.
    for c2 in sorted(cond2):
        df1= Pmean.xs((c2), axis =0, drop_level=False)
        ar1 = np.array(df1)
        #calculates the variance between the replicates p-value means.
        pvar_val = np.nanvar(ar1[:,0])
        #calculates the variance between the replicates u-value means.
        uvar_val = np.nanvar(ar1[:,1])
        #sets information into a dataframe
        name = (df1.index[0][1],df1.index[0][0],df1.index[0][2],pvar_val,uvar_val)
        columns = list(P_var)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        P_var = P_var.append(data, True)
    P_var = P_var.set_index(['Condition','Batch','Plate'])
    P_var_cond = P_var.groupby(level=[0,1]).mean()
    P_var_cond = P_var_cond.reset_index()
    P_var_cond = P_var_cond.rename(columns={'Variance P-Value':'Mean Variance P-Value','Variance U-Stat':'Mean Variance U-Stat'})
    
    P_var_cond.to_csv(outputfile, index = False)
    return P_var_cond

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import os
import pandas as pd
import re

def get_options():

    parser = argparse.ArgumentParser(description="Add the gene names from the plate info files to make the final dataset. The plate info files must be in a folder by themselves and should be .txt files.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The CSV output of S_Scores.py")
    parser.add_argument("-o", "--Outputpath", help="A path a prefix for the output files E.g ~/Desktop/ChemGAPP would make ChemGAPP_Final_Dataset.txt .")
    parser.add_argument("-p", "--PATH", help="The path to the folder containing the plate info files.")
    return parser.parse_args()

def main():
    options = get_options()
    PATH = options.PATH
    inputfile = options.InputFile
    outputpath = options.Outputpath
    
    def plate_info(file):
        p = pd.read_table(file)
        #renames the plate info file columns and adds row and column to index before sorting by index.
        if len(p.columns) == 3:
            p.columns = ['row','column','strain']
            p = p.set_index(['row','column'])
            p = p.sort_index()
        else:
            print("Plate information file" + str(file) + "not in correct format.")
        return p
    # assign directory
    directory = os.path.expanduser(PATH)
    # iterate over files in
    # that directory
    files = []
    for filename in os.listdir(directory):
        f = os.path.join(directory, filename)
        # checking if it is a file
        if f.endswith(".txt"):
            if os.path.isfile(f):
                #print(f)
                files.append(f)
    
    # looks for digits within the file names so that order of 
    # plate info files is sorted plate1, plate2, plate3 etc. not plate1, plate10, plate2 etc.
    def atoi(text):
        return int(text) if text.isdigit() else text

    def natural_keys(text):
        return [ atoi(c) for c in re.split(r'(\d+)', text) ]

    files.sort(key=natural_keys)
    plate_DFs = []
    for i in files:
        p = plate_info(i)
        plate_DFs.append(p)
    
    nm3 = pd.read_csv(inputfile,index_col=[0, 1], header=[0, 1])
    plates = {x[0] for x in nm3.columns}
    nm4 = nm3.copy(deep=False)
    nm4['1','strain'] = 'p'
    columns1 = {x[1] for x in nm4.columns}
    #names columns with condition
    df_with_strains = pd.DataFrame(columns = sorted(columns1))
    # splits by each source plate and then assigns gene names from matching plate information file.
    for a, n in zip(plate_DFs, sorted(plates)):
        df1 = (nm3.xs((n), axis =1, drop_level=True))
        df2 = pd.merge(df1,a,left_index=True, right_index=True)
        df_with_strains = pd.concat([df_with_strains , df2], ignore_index=True)
    df_with_strains = df_with_strains.rename(columns={'strain': 'Gene'})
    df_with_strains = df_with_strains.set_index('Gene')

    #averages scores for rows with the same gene name.
    groupmean = df_with_strains.groupby(level=0).mean()
    df_averaged = pd.DataFrame(groupmean, index=groupmean.index, columns=groupmean.columns)
    df_averaged.index.name = None
    outputfile = os.path.expanduser(outputpath)
    df_with_strains.to_csv(outputfile+"_Final_Dataset.txt", sep='\t')
    df_averaged.to_csv(outputfile+"_Final_Dataset_Averaged.txt", sep='\t')
    return df_with_strains

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import scipy.stats as stats

def get_options():
    parser = argparse.ArgumentParser(description="The variance of replicate colony sizes is calculated for each plate and these variance values are averaged for each plate within a condition.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The normalised csv file from Check_Normalisation.py")
    parser.add_argument("-o", "--OutputFile", help="A CSV file of the average variances for each condition.")
    return parser.parse_args()


def main():
    options = get_options()
    inputfile = options.InputFile
    outputfile = options.OutputFile
    m = pd.read_csv(inputfile,
                          index_col=[0, 1],
                          header=[0, 1, 2,3])
    m.columns = m.columns.swaplevel(2, 3)
    #makes df with same index as the input normalised dataset file.
    Var_DF = pd.DataFrame(index=m.index)
    conditions = {x[0:3] for x in m.columns}
    rounds = 0 
    #splits into source plate, batch and condition, then compares the variance between the replicates.
    for c in sorted(conditions):
        rounds = rounds + 1
        print(rounds)
        df1 = m.xs((c), axis =1, drop_level=False)
        ar1 = np.array(df1)
        ar2 = np.array([])
        for j in range(0,len(ar1)):
            #The variance of each row is calculated and added to the variance column 
            #if all values are nan then variance is set to nan.
            if np.count_nonzero(~np.isnan(ar1[j])) == 0:
                var = "nan"
            #otherwise calculates variance ingnoring nans
            else:
                var = np.nanvar(ar1[j])
            #appends variance to array of variances
            ar2 = np.append(ar2, var)
        #set array as df
        ar_df = pd.DataFrame(ar2, index=m.index)
        #appends array of variances to variance df
        Var_DF = pd.concat([Var_DF, ar_df], axis=1)
    #sets column names to the source plate, batch and condition.
    Var_DF.columns = (pd.MultiIndex.from_tuples(sorted(conditions)))
    ave_Var_plate = pd.DataFrame(columns=['Condition','Batch','Plate','Average Variance'])
    #calculates the average variance for each condition.
    for f in Var_DF.columns:
        name = (f[1],f[2],f[0],np.nanmean(Var_DF[f].values.astype(float)))
        columns = list(ave_Var_plate)
        data = []
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        ave_Var_plate = ave_Var_plate.append(data, True)
    ave_Var_plate = ave_Var_plate.set_index(['Condition',"Batch",'Plate'])
    cond3 = {x[0:2] for x in ave_Var_plate.index}
    #calculates mean across different source plates and produces df.
    ave_Var_cond = pd.DataFrame(columns=(['Condition','Batch','Average Variance']))
    for cd3 in sorted(cond3):
        dfVC = ave_Var_plate.xs(cd3, axis =0, drop_level=False)
        name = (cd3[0],cd3[1],dfVC['Average Variance'].mean())
        data = []
        columns = list(ave_Var_cond)
        zipped = zip(columns, name)
        a_dictionary = dict(zipped)
        data.append(a_dictionary)
        ave_Var_cond = ave_Var_cond.append(data, True)
    ave_Var_cond.to_csv(outputfile,index=False)
    return ave_Var_cond

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def get_options():
    parser = argparse.ArgumentParser(description="Produces a bar plot showing the counts of conditions with a certain number of plates lost at different thresholds of normality (z-score) and Mann-Whitney p-value.",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-i", "--InputFile", help="The input file for the module. Uses the merged output file from Pass_Fail_Plates.py")
    parser.add_argument("-o", "--OutputPlot", help="Name of output file, a PDF of the bar chart")
    return parser.parse_args()

def main():
    options = get_options()
    pass_fail_file = options.InputFile
    otptplot = options.OutputPlot
    Pass_Fail = pd.read_csv(pass_fail_file)
    Pass_Fail['Plate'] = pd.to_numeric(Pass_Fail['Plate'])
    Pass_Fail = Pass_Fail.set_index(['Condition','Plate'])
    plt5 = {x[0:2] for x in Pass_Fail.index}
    bar = pd.DataFrame(columns = ['Condition','Plate','Batch','Replicates', 'Threshold','Percentage'])
    for p5 in sorted(plt5):
        dfpf14 = Pass_Fail.xs((p5), axis=0, drop_level=False)
        #splits the data by condition and source plate number so number of F's 
        # can be counted for each threshold across replicates to produce percentage
        # fails by dividing by overall count and multiplying by 100. This then allows
        # the plotting of bar charts with percentage of plates within a condition failing at certain thresholds
        for cols in dfpf14.columns:
            if cols[0] == 'M' or cols[0] == 'N':
                failed = ((len(dfpf14[dfpf14[cols] == 'F'])/dfpf14[cols].count())*100)
                name=(dfpf14.index[0][0],dfpf14.index[0][1],dfpf14['Batch'][0],dfpf14['Replicates within Condition/Plate'][0],cols,failed)
                columns = list(bar)
                data = []
                zipped = zip(columns, name)
                a_dictionary = dict(zipped)
                data.append(a_dictionary)
                bar = bar.append(data, True) 
        bar = bar.round(2)
    bar_list = bar['Percentage'].astype(float).values
    b3 = {x for x in bar_list}
    b3 = list(sorted(b3))
    bar['Percentage'] = pd.Categorical(bar['Percentage'], categories=b3,ordered=True)
    sns.set_theme(style="whitegrid")
    #stacks the percentages so you can see the how much each percentage contributes at different thresholds.
    b = sns.displot(bar, x='Threshold', hue='Percentage',
                    multiple='stack', legend = True, palette=sns.color_palette("Spectral_r",len(b3)))
    b.set_axis_labels("Threshold Types", "Count of Condition With % Plates Lost")
    b.set_xticklabels(rotation=90)
    plt.title("Count of Conditions with Different Percentages of Plates Lost at Different Thresholds")
    plt.savefig(otptplot, bbox_inches='tight')
    return dfpf14

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# coding: utf-8

# In[ ]:
import argparse
import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import os
from bioinfokit.analys import stat
from statannotations.Annotator import Annotator

def get_options():
    parser = argparse.ArgumentParser(description="Produces barplot displaying genetic interaction type.",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-p", "--PATH", help="Path to Interaction Score Files")
    parser.add_argument("-o", "--OutputFile", help="A PDF file of the final bar plot.")
    parser.add_argument("-g", "--PrimaryGene", help="The primary interacting gene being compared")
    return parser.parse_args()
    

def GI_BarPlot():
    options = get_options()
    outputfile = options.OutputFile
    Path = options.PATH
    primary_gene = options.PrimaryGene
    directory = Path
    files = []
    #assigns all interaction score files within the the directory to a list.
    for filename in os.listdir(directory):
        f = os.path.join(directory, filename)
        # checking if it is a file
        if f.endswith("Interaction_Scores.csv"):
            if os.path.isfile(f):
                print((f.split('/')[-1].split("_Interaction_Scores.csv")[0]))
                files.append(f)
    m_s = None          
    for a in files:
        #adds the name of the secondary gene into a column. 
        if m_s is None:
            m_s = pd.read_csv(a,index_col=[0])
            m_s['Gene'] = a.split('/')[-1].split("_Interaction_Scores.csv")[0]
        else:
            m_s2 = pd.read_csv(a,index_col=[0])
            m_s2['Gene'] = a.split('/')[-1].split("_Interaction_Scores.csv")[0]
            m_s = pd.concat([m_s,m_s2], axis=0)
    
    secondary_genes = set(m_s.Gene.values)
    # performs ANOVA and Tukey HSD between the expected and experimental double knockout fitness ratios
    p_value_list = []
    pairs_list = []
    for i in sorted(secondary_genes):
        df1 = m_s[m_s['Gene'] == i]
        df1_doub = df1[["Double Expected","Double Observed"]]
        df1_doub = df1_doub.melt()
        res = stat()
        res.tukey_hsd(df=df1_doub, res_var='value', xfac_var='variable', anova_model='value ~ C(variable)')
        stats1 = res.tukey_summary
        pairs = list(zip(stats1.group1, stats1.group2))
        pairs_list.append(pairs)
        pvalues = list(stats1['p-value'])
        p_value_list.append(pvalues)
    #plots the bar plots, split by the secondary gene
    sns.set_theme(style="white")
    g = sns.catplot(data=m_s,kind="bar", alpha=0.8, height=6,col="Gene",col_wrap=15,aspect=.4,capsize=.15,errwidth=0.5)
    g.set_xticklabels([])
    def change_width(ax, new_value) :
        for patch in ax.patches :
            current_width = patch.get_width()
            diff = current_width - new_value

            # we change the bar width
            patch.set_width(new_value)

            # we recenter the bar
            patch.set_x(patch.get_x() + diff * .5)

    for ax in g.axes:
        change_width(ax, 1)
        for p,i in zip(ax.patches,range(len(ax.patches))):
                if i == 0:
                    p.set_color('coral')
                if i == 1:
                    p.set_color('peachpuff')
                if i == 2:
                    p.set_color('olive')
                if i == 3:
                    p.set_color('lightseagreen')
    #annotates the anova p-values for each secondary gene plot.
    for ax,pairs,pvalues in zip(g.axes,pairs_list,p_value_list):
        boxax = sns.boxplot(showmeans=False,
                meanline=False,
                medianprops={'visible': False},
                whiskerprops={'visible': False},whis=0,
                data=m_s,
                showfliers=False,
                showbox=False,
                showcaps=False,
                ax=ax)
        plt.xticks([])
        annotator = Annotator(boxax, pairs, data=m_s)
        annotator.set_pvalues(pvalues)
        annotator.configure(line_width = 1)
        annotator.annotate()
    g.set_titles("{col_name}",size=15,y=-0.08)
    #add legend to plot with matching colours
    eight = mlines.Line2D([], [], color='coral', marker='s', ls='', label=primary_gene,ms=15)
    nine = mlines.Line2D([], [], color='peachpuff', marker='s', ls='', label="Secondary Gene",ms=15)
    ten = mlines.Line2D([], [], color='olive', marker='s', ls='', label="Double Expected",ms=15)
    eleven = mlines.Line2D([], [], color='lightseagreen', marker='s', ls='', label="Double Observed",ms=15)
    g.axes.flat[0].set_ylabel("Fitness Ratio", fontsize=15)
    plt.legend(handles=[eight,nine,ten,eleven],
               loc='center left', bbox_to_anchor=(1, 0.5),
          fancybox=True, shadow=False, ncol=1,fontsize=14)
    plt.subplots_adjust(hspace=0.4, wspace=0.05)
    plt.savefig(outputfile, bbox_inches='tight')

if __name__ == "__main__":
    main()